/*============================================================================
  Summary:  Demonstrates Logging in In-Memory OLTP
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more information about SQL Server, check out my website
    http://www.SQLpassion.at
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE master
GO

-- Create new database
CREATE DATABASE InMemoryOLTP
GO

--Add MEMORY_OPTIMIZED_DATA filegroup to the database.
ALTER DATABASE InMemoryOLTP
ADD FILEGROUP InMemoryOLTPFileGroup CONTAINS MEMORY_OPTIMIZED_DATA
GO

USE InMemoryOLTP
GO

-- Add a new file to the previously created file group
ALTER DATABASE InMemoryOLTP ADD FILE
(
	NAME = N'InMemoryOLTPContainer', 
	FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL12.SQL2014\MSSQL\DATA\InMemoryOLTPContainer'
)
TO FILEGROUP [InMemoryOLTPFileGroup]
GO

-- =================================
-- Demonstrates the reduced logging
-- =================================

-- Create a Disk Based table
CREATE TABLE TestTable_DiskBased
(
	Col1 INT NOT NULL PRIMARY KEY,
	Col2 VARCHAR(100) NOT NULL INDEX idx_Col2 NONCLUSTERED,
	Col3 VARCHAR(100) NOT NULL
)
GO

-- Insert 10000 records into the table
DECLARE @i INT = 0

BEGIN TRANSACTION
WHILE (@i < 10000)
BEGIN
	INSERT INTO TestTable_DiskBased VALUES (@i, @i, @i)

	SET @i += 1 
END
COMMIT
GO

-- SQL Server logged more than 20000 log records, because we have 2 indexes
-- defined on the table (Clustered Index, Non-Clustered Index)
SELECT * FROM sys.fn_dblog(NULL, NULL)
WHERE PartitionId IN
(
	SELECT partition_id FROM sys.partitions
	WHERE object_id = OBJECT_ID('TestTable_DiskBased')
)
GO

-- Creates a Memory Optimized table
CREATE TABLE TestTable_MemoryOptimized
(
	Col1 INT NOT NULL PRIMARY KEY NONCLUSTERED HASH WITH (BUCKET_COUNT = 16384),
	Col2 VARCHAR(100) COLLATE Latin1_General_100_Bin2 NOT NULL INDEX idx_Col2,
	Col3 VARCHAR(100) COLLATE Latin1_General_100_Bin2 NOT NULL INDEX idx_Col3
) WITH
(
	MEMORY_OPTIMIZED = ON, 
	DURABILITY = SCHEMA_AND_DATA
)
GO

-- Copy out the highest 'Current LSN'
SELECT * FROM sys.fn_dblog(NULL, NULL)
GO

-- Insert 10000 records into the table
DECLARE @i INT = 0

BEGIN TRANSACTION
WHILE (@i < 10000)
BEGIN
	INSERT INTO TestTable_MemoryOptimized VALUES (@i, @i, @i)

	SET @i += 1 
END
COMMIT
GO

-- Just a few log records!
SELECT * FROM sys.fn_dblog(NULL, NULL)
WHERE [Current LSN] > '00000036:00000013:0028' -- Highest LSN from above
GO

-- Let's look into a LOP_HK log record
SELECT * FROM sys.fn_dblog_xtp(NULL, NULL)
WHERE [Current LSN] > '00000036:00000013:0028' AND Operation = 'LOP_HK'
GO

-- Clean up
USE master
GO

DROP DATABASE InMemoryOLTP
GO